/*
 * $Id: nvidia.h,v 1.12 2003/12/15 15:10:16 lunakl Exp $
 *
 * nVIDIA KWin client (version 0.8)
 *
 * Copyright (C) 2002 Fredrik H�lund <fredrik@kde.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#ifndef __NVIDIA_H
#define __NVIDIA_H

#include <qbutton.h>
#include <kdecoration.h>
#include <kdecorationfactory.h>
#include <kpixmap.h>
#include "tiles.h"

class QSpacerItem;

namespace nVIDIA {

	enum TilePixmap  { TitleLeft=0, TitleCenter, TitleRight,
	                   CaptionSmallLeft, CaptionSmallCenter, CaptionSmallRight,
	                   CaptionLargeLeft, CaptionLargeCenter, CaptionLargeRight,
					   GrabBarLeft, GrabBarCenter, GrabBarRight,
	                   BorderLeft, BorderRight, NumTiles };

	enum Button      { MenuButton=0, OnAllDesktopsButton, HelpButton, MinButton,
	                   MaxButton, CloseButton, NumButtons };

	enum ButtonDeco  { Menu=0, OnAllDesktops, NotOnAllDesktops, Help, Minimize, Maximize,
	                   Restore, Close, NumButtonDecos };	
	enum TileButtons {ButtonRound=0, ButtonSquare, NumButton};

	struct SettingsCache
	{
		bool largeGrabBars:1;
		bool smallCaptionBubbles:1;
	};
	class nVIDIAHandler : public KDecorationFactory
	{
		public:
			nVIDIAHandler();
			~nVIDIAHandler();

			virtual QValueList< BorderSize > borderSizes() const;
			virtual bool reset( unsigned long changed );
                        virtual KDecoration* createDecoration( KDecorationBridge* );

			bool showAppIcons() const        { return showIcons; }
			bool useShadowedText() const     { return shadowedText; }
			bool largeCaptionBubbles() const { return !smallCaptionBubbles; }

			int titleBarHeight( bool large ) const {
				return ( large ? activeTiles[CaptionLargeCenter]->height()
						: activeTiles[CaptionSmallCenter]->height() );
			}

			int grabBarHeight() const
				{ return activeTiles[GrabBarCenter]->height(); }

			inline const QPixmap *tile( TilePixmap tilePix, bool active ) const;
                        inline const QPixmap *btile(TileButtons buttonPix, bool active) const;
			inline const QPixmap *decotile(ButtonDeco tiledeco, bool active) const;
			inline const QPixmap *decostate(ButtonDeco decostate, bool active, bool state) const;
		private:
			void readConfig();
			void createPixmaps();
			void destroyPixmaps();

			void addWidth  (int width,  QPixmap *&pix, bool left, QPixmap *bottomPix);
			void addHeight (int height, QPixmap *&pix);
			void flip( QPixmap *&, QPixmap *& );
			void flip( QPixmap *& );
			void pretile( QPixmap *&, int, Qt::Orientation );
			QPixmap *composite( QImage *, QImage * );
			QImage  *loadImage( const QString &);//, const QColor & );
			QPixmap *loadPixmap( const QString &);//, const QColor & );

			bool showIcons:1, shadowedText:1,
				smallCaptionBubbles:1, largeGrabBars:1;
			SettingsCache *settings_cache;
			nVIDIAImageDb *imageDb;

			QPixmap *activeTiles[ NumTiles ];
			QPixmap *inactiveTiles[ NumTiles ];
			QPixmap *activebuttonDecos[ NumButtonDecos ];
			QPixmap *inactivebuttonDecos[NumButtonDecos]; 
			QPixmap *overbuttonDecos[ NumButtonDecos ];
			QPixmap *pressedbuttonDecos[ NumButtonDecos ];
			QPixmap *inoverbuttonDecos[ NumButtonDecos ];
			QPixmap *inpressedbuttonDecos[ NumButtonDecos ];
                        QPixmap *activeButtons[NumButton];
                        QPixmap *inactiveButtons[NumButton];
	}; // class nVIDIAHandler

	class nVIDIAClient;
	class nVIDIAButton : public QButton
	{
		public:
			nVIDIAButton( nVIDIAClient *, const char *, Button, const QString &, const int realizeBtns = LeftButton );
			~nVIDIAButton();

			int lastButton() const { return lastbutton; }

		private:
			void enterEvent( QEvent * );
			void leaveEvent( QEvent * );
			void mousePressEvent( QMouseEvent * );
			void mouseReleaseEvent( QMouseEvent * );
			void drawButton( QPainter * );

		private:
			nVIDIAClient *client;
			Button button;
			bool hover;
			int lastbutton;
			int realizeButtons;
	}; // class nVIDIAButton


	class nVIDIAClient : public KDecoration
	{
		Q_OBJECT

		public:

			nVIDIAClient( KDecorationBridge* bridge, KDecorationFactory* factory );
			~nVIDIAClient();
                        virtual void init();
			virtual void reset( unsigned long changed );
			virtual Position mousePosition( const QPoint& p ) const;
		    	virtual void borders( int& left, int& right, int& top, int& bottom ) const;
			virtual void resize( const QSize& s );
			virtual QSize minimumSize() const;
			virtual bool eventFilter( QObject* o, QEvent* e );
			virtual void activeChange();
			virtual void captionChange();
                        virtual void maximizeChange();
                        virtual void desktopChange();
                        virtual void shadeChange() {};

		private:
			void createLayout();
			void addButtons( QBoxLayout*, const QString & );
			void updateMask(); // FRAME
			void updateCaptionBuffer();
			void iconChange();
			void resizeEvent( QResizeEvent *); // FRAME
			void paintEvent( QPaintEvent *); // FRAME
			void mouseDoubleClickEvent( QMouseEvent * ); // FRAME
			int width() const { return widget()->width(); }
			int height() const { return widget()->height(); }

			void calculateCaptionRect();

			inline bool maximizedVertical() const {
				return ( maximizeMode() & MaximizeVertical );
			}

		private slots:
			void menuButtonPressed();
			void slotMaximize();

		private:
			QSpacerItem   *topSpacer, *titlebar;
			nVIDIAButton *button[ NumButtons ];
			QRect          captionRect;
			QPixmap        captionBuffer;
			QPixmap       *activeIcon, *inactiveIcon;
			bool           captionBufferDirty:1, maskDirty:1;
			bool           largeCaption:1, largeTitlebar:1;
	}; // class nVIDIAClient

} // namespace nVIDIA

#endif // ___NVIDIA_H

// vim: set noet ts=4 sw=4:
